/* ********************************************************
 * Water Pressure Level
 * using a 4mA-20mA Current Loop Sensor for 500cm waterdepth
 * https://werner.rothschopf.net/
 *
 * by noiasca
 * 2020-07-19
 * ****************************************************** */

# include <NoiascaCurrentLoop.h>       // download library from https://werner.rothschopf.net/

const byte sensorPin = A2;             // pin for the sensor
const uint16_t resistor = 150;         // pull down resistor in Ohm
const byte vref = 33;                  // VREF in Volt*10  (Uno 16Mhz 50, ProMini 8Mhz 3V3, ) - be carefull, an UNO powered by usb will NOT HAVE 5V on VREF
const uint16_t maxValue = 500;         // max value of sensor at 20mA
//CurrentLoopSensor pressureSensor(sensorPin, resistor, vref, maxValue);

CurrentLoopSensor pressureSensor(sensorPin, 220, 50, 500);           // UNO: Pin A0 | 250 Ohm |  5.0Volt | 500cm

void doMeasurement()
{
  const uint16_t interval = 2000;
  static uint32_t previousMillis = 0 - interval;
  if (millis() - previousMillis > interval)
  {
    int level = pressureSensor.getValue();
    Serial.print("adc=");
    Serial.print(pressureSensor.getAdc());
    Serial.print("\t level=");
    Serial.println(level);
    previousMillis = millis();
  }
}

void setup() {
  Serial.begin(115200);
  Serial.println(F("Start water pressure level"));
  pressureSensor.begin();              // start the sensor
  pressureSensor.check();              // remove this line if check shows no error, will save about 320 bytes program memory (flash)
}

void loop() {
  doMeasurement();
}